@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004, 2005 IBM Corporation and others.
+ * Copyright (c) 2004, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,6 +10,9 @@
  *******************************************************************************/
 package org.eclipse.cdt.internal.core.dom.parser.cpp;
 
+import org.eclipse.cdt.core.CCorePlugin;
+import org.eclipse.cdt.core.dom.IPDOM;
+import org.eclipse.cdt.core.dom.IPDOMResolver;
 import org.eclipse.cdt.core.dom.ast.ASTVisitor;
 import org.eclipse.cdt.core.dom.ast.DOMException;
 import org.eclipse.cdt.core.dom.ast.IASTArrayDeclarator;
@@ -50,6 +53,8 @@ import org.eclipse.cdt.core.dom.ast.cpp.ICPPASTOperatorName;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPASTTranslationUnit;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPNamespace;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPScope;
+import org.eclipse.cdt.core.dom.ast.gnu.cpp.GPPLanguage;
+import org.eclipse.cdt.core.model.ILanguage;
 import org.eclipse.cdt.core.parser.ParserLanguage;
 import org.eclipse.cdt.core.parser.ast.IASTEnumerator;
 import org.eclipse.cdt.core.parser.util.ArrayUtil;
@@ -60,6 +65,8 @@ import org.eclipse.cdt.internal.core.dom.parser.IRequiresLocationInformation;
 import org.eclipse.cdt.internal.core.dom.parser.GCCBuiltinSymbolProvider.CPPBuiltinParameter;
 import org.eclipse.cdt.internal.core.parser.scanner2.ILocationResolver;
 import org.eclipse.cdt.internal.core.parser.scanner2.InvalidPreprocessorNodeException;
+import org.eclipse.cdt.internal.core.pdom.PDOM;
+import org.eclipse.core.runtime.CoreException;
 
 /**
  * @author jcamelon
@@ -74,6 +81,7 @@ public class CPPASTTranslationUnit extends CPPASTNode implements
 
     private ILocationResolver resolver;
 
+    private IPDOM pdom;
 
     private static final IASTPreprocessorStatement[] EMPTY_PREPROCESSOR_STATEMENT_ARRAY = new IASTPreprocessorStatement[0];
 
@@ -87,6 +95,10 @@ public class CPPASTTranslationUnit extends CPPASTNode implements
 
     private static final IASTName[] EMPTY_NAME_ARRAY = new IASTName[0];
 
+    public IASTTranslationUnit getTranslationUnit() {
+    	return this;
+    }
+    
     public void addDeclaration(IASTDeclaration d) {
         decls = (IASTDeclaration [])ArrayUtil.append( IASTDeclaration.class, decls, d );
     }
@@ -175,7 +187,19 @@ public class CPPASTTranslationUnit extends CPPASTNode implements
                 return EMPTY_NAME_ARRAY;
             return resolver.getDeclarations( (IMacroBinding)b );
         }
-        return CPPVisitor.getDeclarations( this, b );
+        IASTName[] names = CPPVisitor.getDeclarations( this, b );
+        if (names.length == 0 && pdom != null) {
+        	try {
+        		b = ((PDOM)pdom).getLinkage(getLanguage()).adaptBinding(b);
+        		if (binding != null)
+        			names = ((IPDOMResolver)pdom.getAdapter(IPDOMResolver.class)).getDeclarations(b);
+        	} catch (CoreException e) {
+        		CCorePlugin.log(e);
+        		return names;
+        	}
+        }
+        
+        return names;
     }
 
     /*
@@ -183,15 +207,32 @@ public class CPPASTTranslationUnit extends CPPASTNode implements
      * 
      * @see org.eclipse.cdt.core.dom.ast.IASTTranslationUnit#getDefinitions(org.eclipse.cdt.core.dom.ast.IBinding)
      */
-    public IASTName[] getDefinitions(IBinding aBinding) {
-        IASTName[] foundDefs = getDeclarations(aBinding);
-        
-        for(int i=0; i<foundDefs.length; i++) {
-            if (!foundDefs[i].isDefinition())
-                foundDefs[i] = null;
+    public IASTName[] getDefinitions(IBinding binding) {
+    	if (binding instanceof IMacroBinding) {
+    		if( resolver == null )
+    			return EMPTY_NAME_ARRAY;
+    		return resolver.getDeclarations((IMacroBinding)binding);
         }
         
-        return (IASTName[])ArrayUtil.removeNulls(IASTName.class, foundDefs);
+        IASTName[] names = CPPVisitor.getDeclarations(this, binding);
+        for (int i = 0; i < names.length; i++) {
+            if (!names[i].isDefinition())
+                names[i] = null;
+        }
+        names = (IASTName[])ArrayUtil.removeNulls(IASTName.class, names);
+        
+        if (names.length == 0 && pdom != null) {
+        	try {
+        		binding = ((PDOM)pdom).getLinkage(getLanguage()).adaptBinding(binding);
+        		if (binding != null)
+        			names = ((IPDOMResolver)pdom.getAdapter(IPDOMResolver.class)).getDefinitions(binding);
+        	} catch (CoreException e) {
+        		CCorePlugin.log(e);
+        		return names;
+        	}
+        }
+        
+        return names;
     }
     
     /*
@@ -563,4 +604,18 @@ public class CPPASTTranslationUnit extends CPPASTNode implements
     public ParserLanguage getParserLanguage() {
         return ParserLanguage.CPP;
     }
+    
+    public ILanguage getLanguage() {
+    	// Assuming gnu C++ for now.
+    	return new GPPLanguage();
+    }
+    
+    public IPDOM getIndex() {
+    	return pdom;
+    }
+    
+    public void setIndex(IPDOM pdom) {
+    	this.pdom = pdom;
+    }
+    
 }
