@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 QNX Software Systems and others.
+ * Copyright (c) 2000, 2006 QNX Software Systems and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,7 @@
  *
  * Contributors:
  *     QNX Software Systems - Initial API and implementation
+ *     Ken Ryall (Nokia) - https://bugs.eclipse.org/bugs/show_bug.cgi?id=118894
  *******************************************************************************/
 package org.eclipse.cdt.debug.internal.core.model;
 
@@ -24,7 +25,7 @@ import org.eclipse.cdt.core.IBinaryParser.ISymbol;
 import org.eclipse.cdt.debug.core.CDIDebugModel;
 import org.eclipse.cdt.debug.core.CDebugCorePlugin;
 import org.eclipse.cdt.debug.core.CDebugUtils;
-import org.eclipse.cdt.debug.core.DebugCoreMessages;
+import org.eclipse.cdt.debug.core.ICDTLaunchConfigurationConstants;
 import org.eclipse.cdt.debug.core.ICDebugConstants;
 import org.eclipse.cdt.debug.core.ICGlobalVariableManager;
 import org.eclipse.cdt.debug.core.cdi.CDIException;
@@ -58,6 +59,7 @@ import org.eclipse.cdt.debug.core.cdi.model.ICDISharedLibrary;
 import org.eclipse.cdt.debug.core.cdi.model.ICDISignal;
 import org.eclipse.cdt.debug.core.cdi.model.ICDITarget;
 import org.eclipse.cdt.debug.core.cdi.model.ICDITargetConfiguration;
+import org.eclipse.cdt.debug.core.cdi.model.ICDITargetConfiguration2;
 import org.eclipse.cdt.debug.core.cdi.model.ICDIThread;
 import org.eclipse.cdt.debug.core.cdi.model.ICDIVariableDescriptor;
 import org.eclipse.cdt.debug.core.model.CDebugElementState;
@@ -73,9 +75,9 @@ import org.eclipse.cdt.debug.core.model.IDebuggerProcessSupport;
 import org.eclipse.cdt.debug.core.model.IDisassembly;
 import org.eclipse.cdt.debug.core.model.IExecFileInfo;
 import org.eclipse.cdt.debug.core.model.IGlobalVariableDescriptor;
+import org.eclipse.cdt.debug.core.model.IModuleRetrieval;
 import org.eclipse.cdt.debug.core.model.IPersistableRegisterGroup;
 import org.eclipse.cdt.debug.core.model.IRegisterDescriptor;
-import org.eclipse.cdt.debug.core.sourcelookup.CDirectorySourceContainer;
 import org.eclipse.cdt.debug.core.sourcelookup.ICSourceLocator;
 import org.eclipse.cdt.debug.core.sourcelookup.ISourceLookupChangeListener;
 import org.eclipse.cdt.debug.internal.core.CBreakpointManager;
@@ -119,6 +121,7 @@ import org.eclipse.debug.core.model.IThread;
 import org.eclipse.debug.core.sourcelookup.ISourceContainer;
 import org.eclipse.debug.core.sourcelookup.ISourceLookupDirector;
 import org.eclipse.debug.core.sourcelookup.ISourceLookupParticipant;
+import org.eclipse.debug.core.sourcelookup.containers.DirectorySourceContainer;
 import org.eclipse.debug.core.sourcelookup.containers.FolderSourceContainer;
 import org.eclipse.debug.core.sourcelookup.containers.ProjectSourceContainer;
 
@@ -267,11 +270,32 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 	}
 
 	public void start( boolean stopInMain, boolean resume ) throws DebugException {
+		String stopSymbol = null;
+		try {
+			if ( stopInMain )
+				stopSymbol = getLaunch().getLaunchConfiguration().getAttribute( ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_STOP_AT_MAIN_SYMBOL, ICDTLaunchConfigurationConstants.DEBUGGER_STOP_AT_MAIN_SYMBOL_DEFAULT );
+		}
+		catch( CoreException e ) {
+			throw new DebugException( e.getStatus() );
+		}
 		ICDITargetConfiguration config = getConfiguration();
 		if ( config.supportsBreakpoints() ) {
 			getBreakpointManager().setInitialBreakpoints();
-			if ( stopInMain ) {
-				stopInMain();
+			if ( stopSymbol != null && stopSymbol.length() != 0 ) {
+				stopAtSymbol( stopSymbol );
+			}
+		}
+		if ( config.supportsResume() && resume ) {
+			resume();
+		}
+	}
+
+	public void start( String stopSymbol, boolean resume ) throws DebugException {
+		ICDITargetConfiguration config = getConfiguration();
+		if ( config.supportsBreakpoints() ) {
+			getBreakpointManager().setInitialBreakpoints();
+			if ( stopSymbol != null && stopSymbol.length() != 0 ) {
+				stopAtSymbol( stopSymbol );
 			}
 		}
 		if ( config.supportsResume() && resume ) {
@@ -342,7 +366,19 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 	}
 
 	protected void initializeModuleManager() {
-		getModuleManager().addModules( new ICModule[] { CModule.createExecutable( this, getExecFile().getPath() ) } );
+		ICDISharedLibrary[] slibs = new ICDISharedLibrary[0];
+		try {
+			slibs = getCDITarget().getSharedLibraries();
+		}
+		catch( CDIException e ) {
+			DebugPlugin.log( e );
+		}
+		ICModule[] modules = new ICModule[slibs.length + 1];
+		modules[0] = CModule.createExecutable( this, getExecFile().getPath() );
+		for ( int i = 0; i < slibs.length; ++i ) {
+			modules[i + 1] = CModule.createSharedLibrary( this, slibs[i] );
+		}
+		getModuleManager().addModules( modules );
 	}
 
 	protected void initializeMemoryBlocks() {
@@ -460,12 +496,15 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 		if ( !canTerminate() ) {
 			return;
 		}
-		changeState( CDebugElementState.TERMINATING );
+		final CDebugElementState newState = CDebugElementState.TERMINATING;		
+		changeState( newState );
 		try {
 			getCDITarget().terminate();
 		}
 		catch( CDIException e ) {
-			restoreOldState();
+			if ( getState() == newState ) {
+				restoreOldState();
+			}
 			targetRequestFailed( e.getMessage(), null );
 		}
 	}
@@ -509,12 +548,15 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 	public void resume() throws DebugException {
 		if ( !canResume() )
 			return;
-		changeState( CDebugElementState.RESUMING );
+		final CDebugElementState newState = CDebugElementState.RESUMING; 
+		changeState( newState );
 		try {
 			getCDITarget().resume( false );
 		}
 		catch( CDIException e ) {
-			restoreOldState();
+			if ( getState() == newState ) {
+				restoreOldState();
+			}
 			targetRequestFailed( e.getMessage(), null );
 		}
 	}
@@ -525,12 +567,15 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 	public void suspend() throws DebugException {
 		if ( !canSuspend() )
 			return;
-		changeState( CDebugElementState.SUSPENDING );
+		final CDebugElementState newState = CDebugElementState.SUSPENDING; 
+		changeState( newState );
 		try {
 			getCDITarget().suspend();
 		}
 		catch( CDIException e ) {
-			restoreOldState();
+			if ( getState() == newState ) {
+				restoreOldState();
+			}
 			targetRequestFailed( e.getMessage(), null );
 		}
 	}
@@ -661,12 +706,15 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 		if ( isDisconnecting() ) {
 			return;
 		}
-		changeState( CDebugElementState.DISCONNECTING );
+		final CDebugElementState newState = CDebugElementState.DISCONNECTING;		
+		changeState( newState );
 		try {
 			getCDITarget().disconnect();
 		}
 		catch( CDIException e ) {
-			restoreOldState();
+			if ( getState() == newState ) {
+				restoreOldState();
+			}
 			targetRequestFailed( e.getMessage(), null );
 		}
 	}
@@ -765,6 +813,8 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 			return getMemoryBlockRetrieval();
 		if ( adapter.equals( IMemoryBlockRetrieval.class ) )
 			return getMemoryBlockRetrieval();
+		if ( adapter.equals( IModuleRetrieval.class ) )
+			return getModuleManager();
 		return super.getAdapter( adapter );
 	}
 
@@ -849,14 +899,24 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 		if ( !canRestart() ) {
 			return;
 		}
-		changeState( CDebugElementState.RESTARTING );
-		ICDILocation location = getCDITarget().createFunctionLocation( "", "main" ); //$NON-NLS-1$ //$NON-NLS-2$
+		String mainSymbol = new String( ICDTLaunchConfigurationConstants.DEBUGGER_STOP_AT_MAIN_SYMBOL_DEFAULT );
+		try {
+			mainSymbol = getLaunch().getLaunchConfiguration().getAttribute( ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_STOP_AT_MAIN_SYMBOL, ICDTLaunchConfigurationConstants.DEBUGGER_STOP_AT_MAIN_SYMBOL_DEFAULT );
+		}
+		catch( CoreException e ) {
+			requestFailed( e.getMessage(), e );
+		}
+		ICDILocation location = getCDITarget().createFunctionLocation( "", mainSymbol ); //$NON-NLS-1$
 		setInternalTemporaryBreakpoint( location );
+		final CDebugElementState newState = CDebugElementState.RESTARTING;
+		changeState( newState );
 		try {
 			getCDITarget().restart();
 		}
 		catch( CDIException e ) {
-			restoreOldState();
+			if ( getState() == newState ) {
+				restoreOldState();
+			}
 			targetRequestFailed( e.getMessage(), e );
 		}
 	}
@@ -987,8 +1047,9 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 		getRegisterManager().targetSuspended();
 		getBreakpointManager().skipBreakpoints( false );
 		List newThreads = refreshThreads();
-		if ( event.getSource() instanceof ICDITarget ) {
-			suspendThreads( event );
+		if (event.getSource() instanceof ICDITarget) {
+			if (!(this.getConfiguration() instanceof ICDITargetConfiguration2) || !((ICDITargetConfiguration2)this.getConfiguration()).supportsThreadControl())
+				suspendThreads(event);
 		}
 		// We need this for debuggers that don't have notifications
 		// for newly created threads.
@@ -1048,7 +1109,10 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 				break;
 		}
 		debugEvents.add( createResumeEvent( detail ) );
-		resumeThreads( debugEvents, detail );
+		
+		if (!(this.getConfiguration() instanceof ICDITargetConfiguration2) || !((ICDITargetConfiguration2)this.getConfiguration()).supportsThreadControl())
+			resumeThreads( debugEvents, detail );
+		
 		fireEventSet( (DebugEvent[])debugEvents.toArray( new DebugEvent[debugEvents.size()] ) );
 	}
 
@@ -1246,7 +1310,7 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 			IBinaryObject file;
 			file = getBinaryFile();
 			if ( file != null ) {
-				fIsLittleEndian = new Boolean( file.isLittleEndian() );
+				fIsLittleEndian = Boolean.valueOf( file.isLittleEndian() );
 			}
 		}
 		return fIsLittleEndian.booleanValue();
@@ -1354,12 +1418,15 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 	public void resumeWithoutSignal() throws DebugException {
 		if ( !canResume() )
 			return;
-		changeState( CDebugElementState.RESUMING );
+		final CDebugElementState newState = CDebugElementState.RESUMING;
+		changeState( newState );
 		try {
 			getCDITarget().resume( false );
 		}
 		catch( CDIException e ) {
-			restoreOldState();
+			if ( getState() == newState ) {
+				restoreOldState();
+			}
 			targetRequestFailed( e.getMessage(), e );
 		}
 	}
@@ -1591,84 +1658,46 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 		return CVariableFactory.createGlobalVariable( this, info, vo );
 	}
 
-	/* (non-Javadoc)
-	 * @see org.eclipse.cdt.debug.core.model.ICDebugTarget#hasModules()
-	 */
-	public boolean hasModules() throws DebugException {
-		CModuleManager mm = getModuleManager();
-		return ( mm != null ) ? mm.hasModules() : false;
-	}
-
-	/* (non-Javadoc)
-	 * @see org.eclipse.cdt.debug.core.model.ICDebugTarget#getModules()
-	 */
-	public ICModule[] getModules() throws DebugException {
-		CModuleManager mm = getModuleManager();
-		return ( mm != null ) ? mm.getModules() : new ICModule[0];
-	}
-
-	/* (non-Javadoc)
-	 * @see org.eclipse.cdt.debug.core.model.ICDebugTarget#loadSymbolsForAllModules()
-	 */
-	public void loadSymbolsForAllModules() throws DebugException {
-		CModuleManager mm = getModuleManager();
-		mm.loadSymbolsForAll();
-	}
-
 	public void sourceContainersChanged( ISourceLookupDirector director ) {
 		setSourceLookupPath( director.getSourceContainers() );
 	}
 
 	private void setSourceLookupPath( ISourceContainer[] containers ) {
-		String[] paths = getSourceLookupPath( containers );
+		ArrayList list = new ArrayList( containers.length );
+		getSourceLookupPath( list, containers );
 		try {
-			getCDITarget().setSourcePaths( paths );
+			getCDITarget().setSourcePaths( (String[])list.toArray( new String[list.size()] ) );
 		}
 		catch( CDIException e ) {
 			CDebugCorePlugin.log( e );
 		}
 	}
 
-	private String[] getSourceLookupPath( ISourceContainer[] containers ) {
-		ArrayList cList = new ArrayList( containers.length );
-		getSourceContainers( cList, containers );
-		ArrayList list = new ArrayList( cList.size() );
-		Iterator it = cList.iterator();
-		while( it.hasNext() ) {
-			ISourceContainer c = (ISourceContainer)it.next();
-			if ( c instanceof ProjectSourceContainer ) {
-				IProject project = ((ProjectSourceContainer)c).getProject();
+	private void getSourceLookupPath( List list, ISourceContainer[] containers ) {
+		for ( int i = 0; i < containers.length; ++i ) {
+			if ( containers[i] instanceof ProjectSourceContainer ) {
+				IProject project = ((ProjectSourceContainer)containers[i]).getProject();
 				if ( project != null && project.exists() )
 					list.add( project.getLocation().toPortableString() );
 			}
-			if ( c instanceof FolderSourceContainer ) {
-				IContainer container = ((FolderSourceContainer)c).getContainer();
+			if ( containers[i] instanceof FolderSourceContainer ) {
+				IContainer container = ((FolderSourceContainer)containers[i]).getContainer();
 				if ( container != null && container.exists() )
 					list.add( container.getLocation().toPortableString() );
 			}
-			if ( c instanceof CDirectorySourceContainer ) {
-				File dir = ((CDirectorySourceContainer)c).getDirectory();
+			if ( containers[i] instanceof DirectorySourceContainer ) {
+				File dir = ((DirectorySourceContainer)containers[i]).getDirectory();
 				if ( dir != null && dir.exists() ) {
 					IPath path = new Path( dir.getAbsolutePath() );
 					list.add( path.toPortableString() );
 				}
 			}
-		}
-		cList.clear();
-		return (String[])list.toArray( new String[list.size()] );
-	}
-
-	private void getSourceContainers( List list, ISourceContainer[] containers ) {
-		for ( int i = 0; i < containers.length; ++i ) {
-			if ( !list.contains( containers[i] ) ) {
-				list.add( containers[i] );
-				if ( containers[i].isComposite() ) {
-					try {
-						getSourceContainers( list, containers[i].getSourceContainers() );
-					}
-					catch( CoreException e ) {
-						CDebugCorePlugin.log( e.getStatus() );
-					}
+			if ( containers[i].isComposite() ) {
+				try {
+					getSourceLookupPath( list, containers[i].getSourceContainers() );
+				}
+				catch( CoreException e ) {
+					CDebugCorePlugin.log( e.getStatus() );
 				}
 			}
 		}
@@ -1713,13 +1742,36 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 		getBreakpointManager().skipBreakpoints( enabled );
 	}
 
-	protected void stopInMain() throws DebugException {
-		ICDILocation location = getCDITarget().createFunctionLocation( "", "main" ); //$NON-NLS-1$ //$NON-NLS-2$
+	protected void stopAtSymbol( String stopSymbol ) throws DebugException {
 		try {
+			ICDILocation location = getCDITarget().createFunctionLocation( "", stopSymbol ); //$NON-NLS-1$ 
 			setInternalTemporaryBreakpoint( location );
 		}
-		catch( DebugException e ) {
-			String message = MessageFormat.format( DebugCoreMessages.getString( "CDebugModel.0" ), new String[]{ e.getStatus().getMessage() } ); //$NON-NLS-1$
+		catch( CoreException e ) {
+			boolean isTerminated = getCDITarget().isTerminated();
+			if ( isTerminated ) {
+				String message = MessageFormat.format( CoreModelMessages.getString( "CDebugTarget.0" ), new String[]{ stopSymbol } ); //$NON-NLS-1$
+				MultiStatus status = new MultiStatus( CDebugCorePlugin.getUniqueIdentifier(), IStatus.OK, message, null );
+				status.add( e.getStatus() );
+				throw new DebugException( status );
+			}
+			String message = MessageFormat.format( CoreModelMessages.getString( "CDebugTarget.2" ), new String[]{ stopSymbol, e.getStatus().getMessage() } ); //$NON-NLS-1$
+			IStatus newStatus = new Status( IStatus.WARNING, e.getStatus().getPlugin(), ICDebugInternalConstants.STATUS_CODE_QUESTION, message, null );
+			if ( !CDebugUtils.question( newStatus, this ) ) {
+				throw new DebugException( new Status( IStatus.OK, e.getStatus().getPlugin(), e.getStatus().getCode(), e.getStatus().getMessage(), null ) );
+			}
+		}
+	}
+
+	protected void stopInMain() throws DebugException {
+		String mainSymbol = new String( ICDTLaunchConfigurationConstants.DEBUGGER_STOP_AT_MAIN_SYMBOL_DEFAULT );
+		try {
+			mainSymbol = getLaunch().getLaunchConfiguration().getAttribute( ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_STOP_AT_MAIN_SYMBOL, ICDTLaunchConfigurationConstants.DEBUGGER_STOP_AT_MAIN_SYMBOL_DEFAULT );
+			ICDILocation location = getCDITarget().createFunctionLocation( "", mainSymbol ); //$NON-NLS-1$ 
+			setInternalTemporaryBreakpoint( location );
+		}
+		catch( CoreException e ) {
+			String message = MessageFormat.format( CoreModelMessages.getString( "CDebugTarget.2" ), new String[]{ mainSymbol, e.getStatus().getMessage() } ); //$NON-NLS-1$
 			IStatus newStatus = new Status( IStatus.WARNING, e.getStatus().getPlugin(), ICDebugInternalConstants.STATUS_CODE_QUESTION, message, null );
 			if ( !CDebugUtils.question( newStatus, this ) ) {
 				terminate();
@@ -1727,4 +1779,24 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 			}
 		}
 	}
+
+	public boolean hasModules() throws DebugException {
+		CModuleManager mm = getModuleManager();
+		if ( mm != null )
+			return mm.hasModules();
+		return false;
+	}
+
+	public ICModule[] getModules() throws DebugException {
+		CModuleManager mm = getModuleManager();
+		if ( mm != null )
+			return mm.getModules();
+		return new ICModule[0];
+	}
+
+	public void loadSymbolsForAllModules() throws DebugException {
+		CModuleManager mm = getModuleManager();
+		if ( mm != null )
+			mm.loadSymbolsForAllModules();
+	}
 }
