@@ -341,7 +341,7 @@ public class SourceManager extends Manager {
 
 			if (isChar && (isSigned || isUnsigned)) {
 				return new CharType(target, typename, isUnsigned);
-			} else if (isShort && (isSigned || isUnsigned)) {
+			} else if (isShort && (isInt || isSigned || isUnsigned)) {
 				return new ShortType(target, typename, isUnsigned);
 			} else if (isInt && (isSigned || isUnsigned)) {
 				return new IntType(target, typename, isUnsigned);
@@ -424,13 +424,15 @@ public class SourceManager extends Manager {
 		Target target = (Target)frame.getTarget();
 		Thread currentThread = (Thread)target.getCurrentThread();
 		StackFrame currentFrame = currentThread.getCurrentStackFrame();
-		target.setCurrentThread(frame.getThread(), false);
-		((Thread)frame.getThread()).setCurrentStackFrame(frame, false);
+		target.lockTarget();
 		try {
+			target.setCurrentThread(frame.getThread(), false);
+			((Thread)frame.getThread()).setCurrentStackFrame(frame, false);
 			return getDetailTypeName(target, variable);
 		} finally {
 			target.setCurrentThread(currentThread, false);
 			currentThread.setCurrentStackFrame(currentFrame, false);
+			target.releaseTarget();
 		}
 	}
 	public String getDetailTypeName(Target target, String typename) throws CDIException {
@@ -460,13 +462,15 @@ public class SourceManager extends Manager {
 		Target target = (Target)frame.getTarget();
 		Thread currentThread = (Thread)target.getCurrentThread();
 		StackFrame currentFrame = currentThread.getCurrentStackFrame();
-		target.setCurrentThread(frame.getThread(), false);
-		((Thread)frame.getThread()).setCurrentStackFrame(frame, false);
+		target.lockTarget();
 		try {
+			target.setCurrentThread(frame.getThread(), false);
+			((Thread)frame.getThread()).setCurrentStackFrame(frame, false);
 			return getTypeName(target, variable);
 		} finally {
 			target.setCurrentThread(currentThread, false);
 			currentThread.setCurrentStackFrame(currentFrame, false);
+			target.releaseTarget();
 		}
 	}
 
