@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 QNX Software Systems and others.
+ * Copyright (c) 2000, 2006 QNX Software Systems and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -59,7 +59,6 @@ import org.eclipse.cdt.debug.mi.core.output.MIStackListArgumentsInfo;
 import org.eclipse.cdt.debug.mi.core.output.MIStackListLocalsInfo;
 import org.eclipse.cdt.debug.mi.core.output.MIVar;
 import org.eclipse.cdt.debug.mi.core.output.MIVarChange;
-import org.eclipse.cdt.debug.mi.core.output.MIVarCreateInfo;
 import org.eclipse.cdt.debug.mi.core.output.MIVarUpdateInfo;
 
 /**
@@ -96,6 +95,7 @@ public class VariableManager extends Manager {
 		Target target = ((Session)getSession()).getTarget(miSession);
 		return getVariable(target, varName);
 	}
+
 	/**
 	 * Return the element that have the uniq varName.
 	 * null is return if the element is not in the cache.
@@ -103,12 +103,16 @@ public class VariableManager extends Manager {
 	public Variable getVariable(Target target, String varName) {
 		Variable[] vars = getVariables(target);
 		for (int i = 0; i < vars.length; i++) {
-			if (vars[i].getMIVar().getVarName().equals(varName)) {
-				return vars[i];
-			}
-			Variable v = vars[i].getChild(varName);
-			if (v != null) {
-				return v;
+			try {
+				if (vars[i].getMIVar().getVarName().equals(varName)) {
+					return vars[i];
+				}
+				Variable v = vars[i].getChild(varName);
+				if (v != null) {
+					return v;
+				}
+			} catch (CDIException e) {
+				// 
 			}
 		}
 		return null;
@@ -171,9 +175,10 @@ public class VariableManager extends Manager {
 			Target target = (Target)frame.getTarget();
 			Thread currentThread = (Thread)target.getCurrentThread();
 			StackFrame currentFrame = currentThread.getCurrentStackFrame();
-			target.setCurrentThread(frame.getThread(), false);
-			((Thread)frame.getThread()).setCurrentStackFrame(frame, false);
+			target.lockTarget();
 			try {
+				target.setCurrentThread(frame.getThread(), false);
+				((Thread)frame.getThread()).setCurrentStackFrame(frame, false);
 				MISession miSession = target.getMISession();
 				RxThread rxThread = miSession.getRxThread();
 				rxThread.setEnableConsole(false);
@@ -192,6 +197,7 @@ public class VariableManager extends Manager {
 				rxThread.setEnableConsole(true);
 				target.setCurrentThread(currentThread, false);
 				currentThread.setCurrentStackFrame(currentFrame, false);
+				target.releaseTarget();
 			}
 		} else {
 			throw new CDIException(CdiResources.getString("cdi.VariableManager.Unknown_type")); //$NON-NLS-1$
@@ -224,9 +230,13 @@ public class VariableManager extends Manager {
 		synchronized (varList) {
 			for (Iterator iterator = varList.iterator(); iterator.hasNext();) {
 				Variable variable = (Variable)iterator.next();
-				if (variable.getMIVar().getVarName().equals(varName)) {
-					iterator.remove();
-					return variable;
+				try {
+					if (variable.getMIVar().getVarName().equals(varName)) {
+						iterator.remove();
+						return variable;
+					}
+				} catch (CDIException e) {
+					//
 				}
 			}
 		}
@@ -359,18 +369,21 @@ public class VariableManager extends Manager {
 			Target target = (Target)argDesc.getTarget();
 			Thread currentThread = (Thread)target.getCurrentThread();
 			StackFrame currentFrame = currentThread.getCurrentStackFrame();
-			target.setCurrentThread(stack.getThread(), false);
-			((Thread)stack.getThread()).setCurrentStackFrame(stack, false);
+			target.lockTarget();
 			try {
+				target.setCurrentThread(stack.getThread(), false);
+				((Thread)stack.getThread()).setCurrentStackFrame(stack, false);
 				MISession mi = target.getMISession();
 				CommandFactory factory = mi.getCommandFactory();
 				MIVarCreate var = factory.createMIVarCreate(name);
-				mi.postCommand(var);
-				MIVarCreateInfo info = var.getMIVarCreateInfo();
-				if (info == null) {
-					throw new CDIException(CdiResources.getString("cdi.Common.No_answer")); //$NON-NLS-1$
-				}
-				argument = new Argument(argDesc, info.getMIVar());
+				mi.postCommand(var, -1);
+				argument = new Argument(argDesc, var);
+//				mi.postCommand(var);
+//				MIVarCreateInfo info = var.getMIVarCreateInfo();
+//				if (info == null) {
+//					throw new CDIException(CdiResources.getString("cdi.Common.No_answer")); //$NON-NLS-1$
+//				}
+//				argument = new Argument(argDesc, info.getMIVar());
 				List variablesList = getVariablesList(target);
 				variablesList.add(argument);
 			} catch (MIException e) {
@@ -378,6 +391,7 @@ public class VariableManager extends Manager {
 			} finally {
 				target.setCurrentThread(currentThread, false);
 				currentThread.setCurrentStackFrame(currentFrame, false);
+				target.releaseTarget();
 			}
 		}
 		return argument;
@@ -388,9 +402,10 @@ public class VariableManager extends Manager {
 		Target target = (Target)frame.getTarget();
 		Thread currentThread = (Thread)target.getCurrentThread();
 		StackFrame currentFrame = currentThread.getCurrentStackFrame();
-		target.setCurrentThread(frame.getThread(), false);
-		((Thread)frame.getThread()).setCurrentStackFrame(frame, false);
+		target.lockTarget();
 		try {
+			target.setCurrentThread(frame.getThread(), false);
+			((Thread)frame.getThread()).setCurrentStackFrame(frame, false);
 			MISession mi = target.getMISession();
 			CommandFactory factory = mi.getCommandFactory();
 			int depth = frame.getThread().getStackFrameCount();
@@ -419,6 +434,7 @@ public class VariableManager extends Manager {
 		} finally {
 			target.setCurrentThread(currentThread, false);
 			currentThread.setCurrentStackFrame(currentFrame, false);
+			target.releaseTarget();
 		}
 		return (ICDIArgumentDescriptor[]) argObjects.toArray(new ICDIArgumentDescriptor[0]);
 	}
@@ -457,12 +473,14 @@ public class VariableManager extends Manager {
 				MISession mi = target.getMISession();
 				CommandFactory factory = mi.getCommandFactory();
 				MIVarCreate var = factory.createMIVarCreate(name);
-				mi.postCommand(var);
-				MIVarCreateInfo info = var.getMIVarCreateInfo();
-				if (info == null) {
-					throw new CDIException(CdiResources.getString("cdi.Common.No_answer")); //$NON-NLS-1$
-				}
-				global = new GlobalVariable(varDesc, info.getMIVar());
+				mi.postCommand(var, -1);
+				global = new GlobalVariable(varDesc, var);
+//				mi.postCommand(var;
+//				MIVarCreateInfo info = var.getMIVarCreateInfo();
+//				if (info == null) {
+//					throw new CDIException(CdiResources.getString("cdi.Common.No_answer")); //$NON-NLS-1$
+//				}
+//				global = new GlobalVariable(varDesc, info.getMIVar());
 				List variablesList = getVariablesList(target);
 				variablesList.add(global);
 			} catch (MIException e) {
@@ -477,9 +495,10 @@ public class VariableManager extends Manager {
 		Target target = (Target)frame.getTarget();
 		Thread currentThread = (Thread)target.getCurrentThread();
 		StackFrame currentFrame = currentThread.getCurrentStackFrame();
-		target.setCurrentThread(frame.getThread(), false);
-		((Thread)frame.getThread()).setCurrentStackFrame(frame, false);
+		target.lockTarget();
 		try {
+			target.setCurrentThread(frame.getThread(), false);
+			((Thread)frame.getThread()).setCurrentStackFrame(frame, false);
 			MISession mi = target.getMISession();
 			CommandFactory factory = mi.getCommandFactory();
 			int level = frame.getLevel();
@@ -502,6 +521,7 @@ public class VariableManager extends Manager {
 		} finally {
 			target.setCurrentThread(currentThread, false);
 			currentThread.setCurrentStackFrame(currentFrame, false);
+			target.releaseTarget();
 		}
 		return (ICDILocalVariableDescriptor[]) varObjects.toArray(new ICDILocalVariableDescriptor[0]);
 	}
@@ -518,18 +538,21 @@ public class VariableManager extends Manager {
 			Target target = (Target)varDesc.getTarget();
 			Thread currentThread = (Thread)target.getCurrentThread();
 			StackFrame currentFrame = currentThread.getCurrentStackFrame();
-			target.setCurrentThread(stack.getThread(), false);
-			((Thread)stack.getThread()).setCurrentStackFrame(stack, false);
+			target.lockTarget();
 			try {
+				target.setCurrentThread(stack.getThread(), false);
+				((Thread)stack.getThread()).setCurrentStackFrame(stack, false);
 				MISession mi = target.getMISession();
 				CommandFactory factory = mi.getCommandFactory();
 				MIVarCreate var = factory.createMIVarCreate(name);
-				mi.postCommand(var);
-				MIVarCreateInfo info = var.getMIVarCreateInfo();
-				if (info == null) {
-					throw new CDIException(CdiResources.getString("cdi.Common.No_answer")); //$NON-NLS-1$
-				}
-				local = new LocalVariable(varDesc, info.getMIVar());
+				mi.postCommand(var, -1);
+				local = new LocalVariable(varDesc, var);
+//				mi.postCommand(var);
+//				MIVarCreateInfo info = var.getMIVarCreateInfo();
+//				if (info == null) {
+//					throw new CDIException(CdiResources.getString("cdi.Common.No_answer")); //$NON-NLS-1$
+//				}
+//				local = new LocalVariable(varDesc, info.getMIVar());
 				List variablesList = getVariablesList(target);
 				variablesList.add(local);
 			} catch (MIException e) {
@@ -537,6 +560,7 @@ public class VariableManager extends Manager {
 			} finally {
 				target.setCurrentThread(currentThread, false);
 				currentThread.setCurrentStackFrame(currentFrame, false);
+				target.releaseTarget();
 			}
 		}
 		return local;
