@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005 QNX Software Systems and others.
+ * Copyright (c) 2005, 2006 QNX Software Systems and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,7 @@
  *
  * Contributors:
  *     QNX Software Systems - initial API and implementation
+ *     Andrew Ferguson (andrew.ferguson@arm.com) - bug 123997
  *******************************************************************************/
 package org.eclipse.cdt.launch;
 
@@ -52,8 +53,6 @@ import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.MultiStatus;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.core.runtime.Status;
-import org.eclipse.core.variables.IStringVariableManager;
-import org.eclipse.core.variables.VariablesPlugin;
 import org.eclipse.debug.core.DebugPlugin;
 import org.eclipse.debug.core.ILaunch;
 import org.eclipse.debug.core.ILaunchConfiguration;
@@ -126,7 +125,7 @@ abstract public class AbstractCLaunchDelegate extends LaunchConfigurationDelegat
 	protected IPath getWorkingDirectoryPath(ILaunchConfiguration config) throws CoreException {
 		String location = config.getAttribute(ICDTLaunchConfigurationConstants.ATTR_WORKING_DIRECTORY, (String)null);
 		if (location != null) {
-			String expandedLocation = getStringVariableManager().performStringSubstitution(location);
+			String expandedLocation = LaunchUtils.getStringVariableManager().performStringSubstitution(location);
 			if (expandedLocation.length() > 0) {
 				return new Path(expandedLocation);
 			}
@@ -190,10 +189,6 @@ abstract public class AbstractCLaunchDelegate extends LaunchConfigurationDelegat
 		return new Path(path);
 	}
 	
-	private static IStringVariableManager getStringVariableManager() {
-		return VariablesPlugin.getDefault().getStringVariableManager();
-	}	
-
 	/**
 	 * @param launch
 	 * @param config
@@ -248,29 +243,16 @@ abstract public class AbstractCLaunchDelegate extends LaunchConfigurationDelegat
 	 * @return the program arguments as a String
 	 */
 	public String getProgramArguments(ILaunchConfiguration config) throws CoreException {
-		String args = config.getAttribute(ICDTLaunchConfigurationConstants.ATTR_PROGRAM_ARGUMENTS, (String)null);
-		if (args != null) {
-			args = getStringVariableManager().performStringSubstitution(args);
-		}
-		return args;
-
+		return LaunchUtils.getProgramArguments(config);
 	}
+
 	/**
 	 * Returns the program arguments as an array of individual arguments.
 	 * 
 	 * @return the program arguments as an array of individual arguments
 	 */
 	public String[] getProgramArgumentsArray(ILaunchConfiguration config) throws CoreException {
-		return parseArguments(getProgramArguments(config));
-	}
-
-	private static String[] parseArguments(String args) {
-		if (args == null)
-			return new String[0];
-		ArgumentParser parser = new ArgumentParser(args);
-		String[] res = parser.parseArguments();
-
-		return res;
+		return LaunchUtils.getProgramArgumentsArray(config);
 	}
 
 	protected ICDebugConfiguration getDebugConfig(ILaunchConfiguration config) throws CoreException {
@@ -311,6 +293,9 @@ abstract public class AbstractCLaunchDelegate extends LaunchConfigurationDelegat
 	}
 
 	// temporary fix for #66015
+	/**
+	 * @deprecated
+	 */
 	protected String renderDebuggerProcessLabel() {
 		String format = "{0} ({1})"; //$NON-NLS-1$
 		String timestamp = DateFormat.getInstance().format(new Date(System.currentTimeMillis()));
@@ -323,7 +308,7 @@ abstract public class AbstractCLaunchDelegate extends LaunchConfigurationDelegat
 	 * @param config
 	 * @return
 	 * @throws CoreException
-	 * @deprecated
+	 * @deprecated Use <code>verifyProgramFile</code> instead.
 	 */
 	protected IFile getProgramFile(ILaunchConfiguration config) throws CoreException {
 		ICProject cproject = verifyCProject(config);
@@ -442,89 +427,6 @@ abstract public class AbstractCLaunchDelegate extends LaunchConfigurationDelegat
 		return null;
 	}
 
-	private static class ArgumentParser {
-
-		private String fArgs;
-		private int fIndex = 0;
-		private int ch = -1;
-
-		public ArgumentParser(String args) {
-			fArgs = args;
-		}
-
-		public String[] parseArguments() {
-			ArrayList v = new ArrayList();
-
-			ch = getNext();
-			while (ch > 0) {
-				while (Character.isWhitespace((char)ch))
-					ch = getNext();
-
-				if (ch == '"') {
-					v.add(parseString());
-				} else {
-					v.add(parseToken());
-				}
-			}
-
-			String[] result = new String[v.size()];
-			v.toArray(result);
-			return result;
-		}
-
-		private int getNext() {
-			if (fIndex < fArgs.length())
-				return fArgs.charAt(fIndex++);
-			return -1;
-		}
-
-		private String parseString() {
-			StringBuffer buf = new StringBuffer();
-			ch = getNext();
-			while (ch > 0 && ch != '"') {
-				if (ch == '\\') {
-					ch = getNext();
-					if (ch != '"') { // Only escape double quotes
-						buf.append('\\');
-					}
-				}
-				if (ch > 0) {
-					buf.append((char)ch);
-					ch = getNext();
-				}
-			}
-
-			ch = getNext();
-
-			return buf.toString();
-		}
-
-		private String parseToken() {
-			StringBuffer buf = new StringBuffer();
-
-			while (ch > 0 && !Character.isWhitespace((char)ch)) {
-				if (ch == '\\') {
-					ch = getNext();
-					if (ch > 0) {
-						if (ch != '"') { // Only escape double quotes
-							buf.append('\\');
-						}
-						buf.append((char)ch);
-						ch = getNext();
-					} else if (ch == -1) { // Don't lose a trailing backslash
-						buf.append('\\');
-					}
-				} else if (ch == '"') {
-					buf.append(parseString());
-				} else {
-					buf.append((char)ch);
-					ch = getNext();
-				}
-			}
-			return buf.toString();
-		}
-	}
-
 	/**
 	 * Recursively creates a set of projects referenced by the current project
 	 * 
@@ -637,44 +539,39 @@ abstract public class AbstractCLaunchDelegate extends LaunchConfigurationDelegat
 	 *             if an exception occurs while checking for compile errors.
 	 */
 	public boolean finalLaunchCheck(ILaunchConfiguration configuration, String mode, IProgressMonitor monitor) throws CoreException {
-		try {
-			boolean continueLaunch = true;
-			if (orderedProjects != null) {
-				monitor.beginTask(LaunchMessages.getString("AbstractCLaunchDelegate.searching_for_errors"), //$NON-NLS-1$
-									orderedProjects.size() + 1);
-
-				boolean compileErrorsInProjs = false;
-
-				//check prerequisite projects for compile errors.
-				for (Iterator i = orderedProjects.iterator(); i.hasNext();) {
-					IProject proj = (IProject)i.next();
-					monitor.subTask(LaunchMessages.getString("AbstractCLaunchDelegate.searching_for_errors_in") //$NON-NLS-1$
-							+ proj.getName());
-					compileErrorsInProjs = existsErrors(proj);
-					if (compileErrorsInProjs) {
-						break;
-					}
-				}
-
-				//check current project, if prerequite projects were ok
-				if (!compileErrorsInProjs) {
-					monitor.subTask(LaunchMessages.getString("AbstractCLaunchDelegate.searching_for_errors_in") //$NON-NLS-1$
-							+ project.getName());
-					compileErrorsInProjs = existsErrors(project);
-				}
-
-				//if compile errors exist, ask the user before continuing.
+		boolean continueLaunch = true;
+		if (orderedProjects != null) {
+			monitor.subTask(LaunchMessages.getString("AbstractCLaunchDelegate.searching_for_errors")); //$NON-NLS-1$
+			
+			boolean compileErrorsInProjs = false;
+			
+			//check prerequisite projects for compile errors.
+			for (Iterator i = orderedProjects.iterator(); i.hasNext();) {
+				IProject proj = (IProject)i.next();
+				monitor.subTask(LaunchMessages.getString("AbstractCLaunchDelegate.searching_for_errors_in") //$NON-NLS-1$
+						+ proj.getName());
+				compileErrorsInProjs = existsErrors(proj);
 				if (compileErrorsInProjs) {
-					IStatusHandler prompter = DebugPlugin.getDefault().getStatusHandler(promptStatus);
-					if (prompter != null) {
-						continueLaunch = ((Boolean)prompter.handleStatus(complileErrorPromptStatus, null)).booleanValue();
-					}
+					break;
+				}
+			}
+			
+			//check current project, if prerequite projects were ok
+			if (!compileErrorsInProjs) {
+				monitor.subTask(LaunchMessages.getString("AbstractCLaunchDelegate.searching_for_errors_in") //$NON-NLS-1$
+						+ project.getName());
+				compileErrorsInProjs = existsErrors(project);
+			}
+			
+			//if compile errors exist, ask the user before continuing.
+			if (compileErrorsInProjs) {
+				IStatusHandler prompter = DebugPlugin.getDefault().getStatusHandler(promptStatus);
+				if (prompter != null) {
+					continueLaunch = ((Boolean)prompter.handleStatus(complileErrorPromptStatus, null)).booleanValue();
 				}
 			}
-			return continueLaunch;
-		} finally {
-			monitor.done();
 		}
+		return continueLaunch;
 	}
 
 	/**
@@ -862,4 +759,4 @@ abstract public class AbstractCLaunchDelegate extends LaunchConfigurationDelegat
 		map.put( IProcess.ATTR_PROCESS_TYPE, ICDTLaunchConfigurationConstants.ID_PROGRAM_PROCESS_TYPE );
 		return map;
 	}
-}
\ No newline at end of file
+}
