@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005 QNX Software Systems and others.
+ * Copyright (c) 2005, 2006 QNX Software Systems and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,7 @@
  *
  * Contributors:
  * QNX Software Systems - Initial API and implementation
+ * Ken Ryall (Nokia) - https://bugs.eclipse.org/bugs/show_bug.cgi?id=118894
  *******************************************************************************/
 package org.eclipse.cdt.launch.ui;
 
@@ -18,7 +19,6 @@ import java.util.Comparator;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-
 import org.eclipse.cdt.core.CCorePlugin;
 import org.eclipse.cdt.core.IBinaryParser;
 import org.eclipse.cdt.core.ICExtensionReference;
@@ -26,10 +26,10 @@ import org.eclipse.cdt.core.IBinaryParser.IBinaryObject;
 import org.eclipse.cdt.debug.core.CDebugCorePlugin;
 import org.eclipse.cdt.debug.core.ICDTLaunchConfigurationConstants;
 import org.eclipse.cdt.debug.core.ICDebugConfiguration;
+import org.eclipse.cdt.debug.core.ICDebugConstants;
 import org.eclipse.cdt.launch.internal.ui.AbstractCDebuggerTab;
 import org.eclipse.cdt.launch.internal.ui.LaunchMessages;
 import org.eclipse.cdt.launch.internal.ui.LaunchUIPlugin;
-import org.eclipse.cdt.launch.internal.ui.PixelConverter;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
@@ -38,18 +38,19 @@ import org.eclipse.core.runtime.Path;
 import org.eclipse.debug.core.ILaunchConfiguration;
 import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
 import org.eclipse.jface.dialogs.Dialog;
-import org.eclipse.jface.dialogs.IDialogConstants;
 import org.eclipse.swt.SWT;
+import org.eclipse.swt.events.ModifyEvent;
+import org.eclipse.swt.events.ModifyListener;
 import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.Button;
-import org.eclipse.swt.widgets.Combo;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Group;
 import org.eclipse.swt.widgets.Shell;
+import org.eclipse.swt.widgets.Text;
 
 public class CDebuggerTab extends AbstractCDebuggerTab {
 
@@ -59,9 +60,6 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 
 		private Button fRegBookKeeping;
 
-		final String[] protocolItems = new String[] { "mi", "mi1", "mi2", "mi3" };   //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$ //$NON-NLS-4$
-		private Combo fPCombo;
-
 		/**
 		 * Constructor for AdvancedDebuggerOptionsDialog.
 		 */
@@ -85,25 +83,10 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 			fVarBookKeeping.setText(LaunchMessages.getString("CDebuggerTab.Variables")); //$NON-NLS-1$
 			fRegBookKeeping = new Button(group, SWT.CHECK);
 			fRegBookKeeping.setText(LaunchMessages.getString("CDebuggerTab.Registers")); //$NON-NLS-1$
-			createProtocolCombo(composite, 2);
 			initialize();
 			return composite;
 		}
 
-		protected void createProtocolCombo(Composite parent, int colspan) {
-			Group comboComp = new Group(parent, SWT.NONE);
-			comboComp.setText("Protocol");
-			GridLayout layout = new GridLayout(2, false);
-			comboComp.setLayout(layout);
-			GridData gd = new GridData(GridData.FILL_HORIZONTAL);
-			gd.horizontalSpan = colspan;
-			comboComp.setLayoutData(gd);
-			fPCombo = new Combo(comboComp, SWT.READ_ONLY | SWT.DROP_DOWN);
-			fPCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
-			fPCombo.setItems(protocolItems);
-		}
-
-
 		protected void okPressed() {
 			saveValues();
 			super.okPressed();
@@ -112,35 +95,18 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 		private void initialize() {
 			Map attr = getAdvancedAttributes();
 			Object varBookkeeping = attr.get(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_ENABLE_VARIABLE_BOOKKEEPING);
-			fVarBookKeeping.setSelection( (varBookkeeping instanceof Boolean) ? ! ((Boolean)varBookkeeping).booleanValue() : true);
+			fVarBookKeeping.setSelection( (varBookkeeping instanceof Boolean) ? !((Boolean)varBookkeeping).booleanValue() : true);
 			Object regBookkeeping = attr.get(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_ENABLE_REGISTER_BOOKKEEPING);
-			fRegBookKeeping.setSelection( (regBookkeeping instanceof Boolean) ? ! ((Boolean)regBookkeeping).booleanValue() : true);
-			Object protocol = attr.get(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_PROTOCOL);
-			int index = 0;
-			if (protocol instanceof String) {
-				String p = (String)protocol;
-				if (p != null && p.length() > 0) {
-					for (int i = 0; i < protocolItems.length; ++i) {
-						if (protocolItems[i].equals(p)) {
-							index = i;
-						}
-					}
-				}
-			}
-			fPCombo.select(index);
+			fRegBookKeeping.setSelection( (regBookkeeping instanceof Boolean) ? !((Boolean)regBookkeeping).booleanValue() : true);
 		}
 
 		private void saveValues() {
 			Map attr = getAdvancedAttributes();
-			Boolean varBookkeeping = (fVarBookKeeping.getSelection()) ? Boolean.FALSE : Boolean.TRUE;
+			Boolean varBookkeeping = Boolean.valueOf( !fVarBookKeeping.getSelection() );
 			attr.put(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_ENABLE_VARIABLE_BOOKKEEPING, varBookkeeping);
-			Boolean regBookkeeping = (fRegBookKeeping.getSelection()) ? Boolean.FALSE : Boolean.TRUE;
+			Boolean regBookkeeping = Boolean.valueOf( !fRegBookKeeping.getSelection() );
 			attr.put(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_ENABLE_REGISTER_BOOKKEEPING, regBookkeeping);
-			String protocol = fPCombo.getText();
-			if (protocol != null && protocol.length() > 0) {
-				attr.put(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_PROTOCOL, protocol);
-			}
-			updateLaunchConfigurationDialog();
+			update();
 		}
 
 		/*
@@ -158,12 +124,19 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 
 	protected Button fAdvancedButton;
 	protected Button fStopInMain;
+	protected Text fStopInMainSymbol;
 	protected Button fAttachButton;
 
 	private Map fAdvancedAttributes = new HashMap(5);
 
 	public CDebuggerTab(boolean attachMode) {
 		fAttachMode = attachMode;
+		// If the default debugger has not been set, use the MI debugger.
+		// The MI plug-in also does this, but it may not have been loaded yet. Bug 158391.
+		ICDebugConfiguration dc = CDebugCorePlugin.getDefault().getDefaultDefaultDebugConfiguration();
+		if (dc == null) {
+			CDebugCorePlugin.getDefault().getPluginPreferences().setDefault(ICDebugConstants.PREF_DEFAULT_DEBUGGER_TYPE, "org.eclipse.cdt.debug.mi.core.CDebuggerNew"); //$NON-NLS-1$
+		}
 	}
 
 	public void createControl(Composite parent) {
@@ -177,7 +150,7 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 		GridData gd = new GridData( GridData.BEGINNING, GridData.CENTER, true, false );
 		comp.setLayoutData(gd);
 
-		createDebuggerCombo( comp, ( fAttachMode ) ? 1 : 2 );
+		createDebuggerCombo(comp, ( fAttachMode ) ? 1 : 2 );
 		createOptionsComposite(comp);
 		createDebuggerGroup(comp, 2);
 	}
@@ -185,7 +158,7 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 	protected void loadDebuggerComboBox(ILaunchConfiguration config, String selection) {
 		ICDebugConfiguration[] debugConfigs;
 		String configPlatform = getPlatform(config);
-		debugConfigs = CDebugCorePlugin.getDefault().getDebugConfigurations();
+		debugConfigs = CDebugCorePlugin.getDefault().getActiveDebugConfigurations();
 		Arrays.sort(debugConfigs, new Comparator() {
 
 			public int compare(Object o1, Object o2) {
@@ -201,6 +174,11 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 		} else {
 			mode = ICDTLaunchConfigurationConstants.DEBUGGER_MODE_RUN;
 		}
+		if (selection.equals("")) { //$NON-NLS-1$
+			ICDebugConfiguration dc = CDebugCorePlugin.getDefault().getDefaultDebugConfiguration();
+			if (dc != null)
+				selection = dc.getID();
+		}
 		String defaultSelection = selection;
 		for (int i = 0; i < debugConfigs.length; i++) {
 			if (debugConfigs[i].supportsMode(mode)) {
@@ -239,6 +217,10 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 		}
 		config.setAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_ENABLE_VARIABLE_BOOKKEEPING, false);
 		config.setAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_ENABLE_REGISTER_BOOKKEEPING, false);
+		ICDebugConfiguration dc = CDebugCorePlugin.getDefault().getDefaultDebugConfiguration();
+		if (dc != null) {
+			config.setAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_ID, dc.getID());
+		}
 	}
 
 	public void initializeFrom(ILaunchConfiguration config) {
@@ -260,8 +242,8 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 					ICDTLaunchConfigurationConstants.DEBUGGER_MODE_ATTACH);
 		} else {
 			config.setAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_STOP_AT_MAIN, fStopInMain.getSelection());
-			config.setAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_START_MODE,
-					ICDTLaunchConfigurationConstants.DEBUGGER_MODE_RUN);
+			config.setAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_STOP_AT_MAIN_SYMBOL, fStopInMainSymbol.getText());
+			config.setAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_START_MODE, ICDTLaunchConfigurationConstants.DEBUGGER_MODE_RUN);
 		}
 		applyAdvancedAttributes(config);
 	}
@@ -278,6 +260,14 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 			setErrorMessage(MessageFormat.format(LaunchMessages.getString("CDebuggerTab.Mode_not_supported"), new String[]{mode})); //$NON-NLS-1$
 			return false;
 		}
+		if ( fStopInMain != null && fStopInMainSymbol != null ) {
+			// The "Stop on startup at" field must not be empty
+			String mainSymbol = fStopInMainSymbol.getText().trim();
+			if (fStopInMain.getSelection() && mainSymbol.length() == 0) {
+				setErrorMessage( LaunchMessages.getString("CDebuggerTab.Stop_on_startup_at_can_not_be_empty")); //$NON-NLS-1$
+				return false;
+			}
+		}
 		if (super.isValid(config) == false) {
 			return false;
 		}
@@ -312,28 +302,36 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 			programName = config.getAttribute(ICDTLaunchConfigurationConstants.ATTR_PROGRAM_NAME, (String)null);
 		} catch (CoreException e) {
 		}
-		IPath exePath = new Path(programName);
-		if (projectName != null && !projectName.equals("")) { //$NON-NLS-1$
-			IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);
-			ICExtensionReference[] parserRef = CCorePlugin.getDefault().getBinaryParserExtensions(project);
-			for (int i = 0; i < parserRef.length; i++) {
-				try {
-					IBinaryParser parser = (IBinaryParser)parserRef[i].createExtension();
-					IBinaryObject exe = (IBinaryObject)parser.getBinary(exePath);
-					if (exe != null) {
-						return exe;
+		if (programName != null ) {
+			IPath exePath = new Path(programName);
+			if (projectName != null && !projectName.equals("")) { //$NON-NLS-1$
+				IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);
+				if (!project.isAccessible()) {
+					return null;
+				}
+				if (!exePath.isAbsolute()) {
+					exePath = project.getLocation().append(exePath);
+				}
+				ICExtensionReference[] parserRef = CCorePlugin.getDefault().getBinaryParserExtensions(project);
+				for (int i = 0; i < parserRef.length; i++) {
+					try {
+						IBinaryParser parser = (IBinaryParser)parserRef[i].createExtension();
+						IBinaryObject exe = (IBinaryObject)parser.getBinary(exePath);
+						if (exe != null) {
+							return exe;
+						}
+					} catch (ClassCastException e) {
+					} catch (IOException e) {
 					}
-				} catch (ClassCastException e) {
-				} catch (IOException e) {
 				}
 			}
-		}
-		IBinaryParser parser = CCorePlugin.getDefault().getDefaultBinaryParser();
-		try {
-			IBinaryObject exe = (IBinaryObject)parser.getBinary(exePath);
-			return exe;
-		} catch (ClassCastException e) {
-		} catch (IOException e) {
+			IBinaryParser parser = CCorePlugin.getDefault().getDefaultBinaryParser();
+			try {
+				IBinaryObject exe = (IBinaryObject)parser.getBinary(exePath);
+				return exe;
+			} catch (ClassCastException e) {
+			} catch (IOException e) {
+			}
 		}
 		return null;
 	}
@@ -358,13 +356,15 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 	/**
 	 * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#updateLaunchConfigurationDialog()
 	 */
-	protected void updateLaunchConfigurationDialog() {
-		super.updateLaunchConfigurationDialog();
+	protected void update() {
+		if (!isInitializing()) {
+			super.updateLaunchConfigurationDialog();
+		}
 	}
 
 	protected void createOptionsComposite(Composite parent) {
 		Composite optionsComp = new Composite(parent, SWT.NONE);
-		int numberOfColumns = (fAttachMode) ? 1 : 2;
+		int numberOfColumns = (fAttachMode) ? 1 : 3;
 		GridLayout layout = new GridLayout( numberOfColumns, false );
 		optionsComp.setLayout( layout );
 		optionsComp.setLayoutData( new GridData( GridData.BEGINNING, GridData.CENTER, true, false, 1, 1 ) );
@@ -373,9 +373,17 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 			fStopInMain.addSelectionListener(new SelectionAdapter() {
 
 				public void widgetSelected(SelectionEvent e) {
-					if (!isInitializing()) {
-						updateLaunchConfigurationDialog();
-					}
+					fStopInMainSymbol.setEnabled(fStopInMain.getSelection());
+					update();
+				}
+			});
+			fStopInMainSymbol = new Text(optionsComp, SWT.SINGLE | SWT.BORDER);
+			final GridData gridData = new GridData(GridData.FILL, GridData.CENTER, false, false);
+			gridData.widthHint = 100;
+			fStopInMainSymbol.setLayoutData(gridData);
+			fStopInMainSymbol.addModifyListener(new ModifyListener() {
+				public void modifyText(ModifyEvent evt) {
+					update();
 				}
 			});
 		}
@@ -412,11 +420,6 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 			attr.put(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_ENABLE_REGISTER_BOOKKEEPING, regBookkeeping);
 		} catch (CoreException e) {
 		}
-		try {
-			String protocol = config.getAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_PROTOCOL, "mi"); //$NON-NLS-1$
-			attr.put(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_PROTOCOL, protocol);
-		} catch (CoreException e) {
-		}
 	}
 
 	private void applyAdvancedAttributes(ILaunchConfigurationWorkingCopy config) {
@@ -429,13 +432,6 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 		if (regBookkeeping instanceof Boolean)
 			config.setAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_ENABLE_REGISTER_BOOKKEEPING,
 					((Boolean)regBookkeeping).booleanValue());
-		Object protocol = attr.get(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_PROTOCOL);
-		if (protocol instanceof String) {
-			String p = (String)protocol;
-			if (p != null && p.length() > 0) {
-				config.setAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_PROTOCOL, p);
-			}
-		}
 	}
 
 	protected Shell getShell() {
@@ -457,6 +453,9 @@ public class CDebuggerTab extends AbstractCDebuggerTab {
 			if (!fAttachMode) {
 				fStopInMain.setSelection(config.getAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_STOP_AT_MAIN,
 						ICDTLaunchConfigurationConstants.DEBUGGER_STOP_AT_MAIN_DEFAULT));
+				fStopInMainSymbol.setText(config.getAttribute(ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_STOP_AT_MAIN_SYMBOL,
+						ICDTLaunchConfigurationConstants.DEBUGGER_STOP_AT_MAIN_SYMBOL_DEFAULT));
+				fStopInMainSymbol.setEnabled(fStopInMain.getSelection());
 			}
 			initializeAdvancedAttributes(config);
 		} catch (CoreException e) {
