@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2002, 2005 IBM Software Corporation and others.
+ * Copyright (c) 2002, 2006 IBM Software Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -18,8 +18,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.Vector;
 
-import org.eclipse.cdt.core.CCProjectNature;
-import org.eclipse.cdt.core.CProjectNature;
 import org.eclipse.cdt.core.model.CModelException;
 import org.eclipse.cdt.core.model.CoreModel;
 import org.eclipse.cdt.core.model.ICProject;
@@ -34,12 +32,15 @@ import org.eclipse.cdt.managedbuilder.core.IBuildObject;
 import org.eclipse.cdt.managedbuilder.core.IBuilder;
 import org.eclipse.cdt.managedbuilder.core.IConfiguration;
 import org.eclipse.cdt.managedbuilder.core.IEnvVarBuildPath;
+import org.eclipse.cdt.managedbuilder.core.IHoldsOptions;
 import org.eclipse.cdt.managedbuilder.core.IManagedBuildInfo;
 import org.eclipse.cdt.managedbuilder.core.IManagedCommandLineGenerator;
 import org.eclipse.cdt.managedbuilder.core.IManagedCommandLineInfo;
+import org.eclipse.cdt.managedbuilder.core.IManagedConfigElement;
 import org.eclipse.cdt.managedbuilder.core.IManagedProject;
 import org.eclipse.cdt.managedbuilder.core.IOption;
 import org.eclipse.cdt.managedbuilder.core.IOptionApplicability;
+import org.eclipse.cdt.managedbuilder.core.IOptionPathConverter;
 import org.eclipse.cdt.managedbuilder.core.IResourceConfiguration;
 import org.eclipse.cdt.managedbuilder.core.ITarget;
 import org.eclipse.cdt.managedbuilder.core.ITool;
@@ -53,7 +54,7 @@ import org.eclipse.cdt.managedbuilder.internal.macros.OptionContextData;
 import org.eclipse.cdt.managedbuilder.internal.scannerconfig.ManagedBuildCPathEntryContainer;
 import org.eclipse.cdt.managedbuilder.macros.BuildMacroException;
 import org.eclipse.cdt.managedbuilder.macros.IBuildMacroProvider;
-import org.eclipse.cdt.managedbuilder.makegen.IManagedDependencyGenerator;
+import org.eclipse.cdt.managedbuilder.makegen.IManagedDependencyGeneratorType;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
 import org.eclipse.core.runtime.CoreException;
@@ -283,7 +284,7 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 	/* (non-Javadoc)
 	 * @see org.eclipse.cdt.managedbuilder.core.IManagedBuildInfo#getDependencyGenerator(java.lang.String)
 	 */
-	public IManagedDependencyGenerator getDependencyGenerator(String sourceExtension) {
+	public IManagedDependencyGeneratorType getDependencyGenerator(String sourceExtension) {
 		// Find the tool and ask the Managed Build Manager for its dep generator
 		try {
 			if (getDefaultConfiguration() != null) {
@@ -407,7 +408,7 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 	 */
 	public String[] getLibsForConfiguration(String extension) {
 		Vector libs = new Vector();
-		ITool tool = getDefaultConfiguration().getTargetTool();
+		ITool tool = getDefaultConfiguration().calculateTargetTool();
 		if(tool == null)
 			tool = getToolFromOutputExtension(extension);
 			
@@ -434,7 +435,7 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 												"", //$NON-NLS-1$
 												" ", //$NON-NLS-1$
 												IBuildMacroProvider.CONTEXT_OPTION,
-												new OptionContextData(option, getDefaultConfiguration().getToolChain()));
+												new OptionContextData(option, tool));
 										if(resolved != null && resolved.length > 0){
 											for(int k = 0; k < resolved.length; k++){
 												String string = resolved[k];
@@ -670,32 +671,16 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 	 * @see org.eclipse.cdt.core.build.managed.IManagedBuildInfo#getToolFromInputExtension(java.lang.String)
 	 */
 	public ITool getToolFromInputExtension(String sourceExtension) {
-		// Get all the tools for the current config
-		ITool[] tools = getFilteredTools();
-		for (int index = 0; index < tools.length; index++) {
-			ITool tool = tools[index];
-			if (tool.buildsFileType(sourceExtension)) {
-				return tool;
-			}
-		}
-		return null;		
+		IConfiguration config = getDefaultConfiguration();
+		return config.getToolFromInputExtension(sourceExtension);
 	}
 
 	/* (non-Javadoc)
 	 * @see org.eclipse.cdt.core.build.managed.IManagedBuildInfo#getToolFromOutputExtension(java.lang.String)
 	 */
 	public ITool getToolFromOutputExtension(String extension) {
-		// Treat a null argument as an empty string
-		String ext = extension == null ? new String() : extension;
-		// Get all the tools for the current config
-		ITool[] tools = getFilteredTools();
-		for (int index = 0; index < tools.length; index++) {
-			ITool tool = tools[index];
-			if (tool.producesFileType(ext)) {
-				return tool;
-			}
-		}
-		return null;		
+		IConfiguration config = getDefaultConfiguration();
+		return config.getToolFromOutputExtension(extension);
 	}
 	
 	/* (non-Javadoc)
@@ -720,11 +705,34 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 				String cmd = tool.getToolCommand();
 				//try to resolve the build macros in the tool command
 				try{
-					String resolvedCommand = ManagedBuildManager.getBuildMacroProvider().resolveValueToMakefileFormat(cmd,
-							"", //$NON-NLS-1$
-							" ", //$NON-NLS-1$
-							IBuildMacroProvider.CONTEXT_FILE,
-							new FileContextData(inputLocation,outputLocation,null,getDefaultConfiguration().getToolChain()));
+					String resolvedCommand = null;
+					
+					if ((inputLocation != null && inputLocation.toString().indexOf(" ") != -1) || //$NON-NLS-1$
+							(outputLocation != null && outputLocation.toString().indexOf(" ") != -1) ) //$NON-NLS-1$
+					{
+						resolvedCommand = ManagedBuildManager
+								.getBuildMacroProvider().resolveValue(
+										cmd,
+										"", //$NON-NLS-1$
+										" ", //$NON-NLS-1$
+										IBuildMacroProvider.CONTEXT_FILE,
+										new FileContextData(inputLocation,
+												outputLocation, null,
+												tool));
+					}
+
+					else {
+						resolvedCommand = ManagedBuildManager
+								.getBuildMacroProvider()
+								.resolveValueToMakefileFormat(
+										cmd,
+										"", //$NON-NLS-1$
+										" ", //$NON-NLS-1$
+										IBuildMacroProvider.CONTEXT_FILE,
+										new FileContextData(inputLocation,
+												outputLocation, null,
+												tool));
+					}
 					if((resolvedCommand = resolvedCommand.trim()).length() > 0)
 						cmd = resolvedCommand;
 						
@@ -745,7 +753,7 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 	 */
 	public String[] getUserObjectsForConfiguration(String extension) {
 		Vector objs = new Vector();
-		ITool tool = getDefaultConfiguration().getTargetTool();
+		ITool tool = getDefaultConfiguration().calculateTargetTool();
 		if(tool == null)
 			tool = getToolFromOutputExtension(extension);
 			
@@ -765,7 +773,7 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 												"", //$NON-NLS-1$
 												" ", //$NON-NLS-1$
 												IBuildMacroProvider.CONTEXT_OPTION,
-												new OptionContextData(option, getDefaultConfiguration().getToolChain()));
+												new OptionContextData(option, tool));
 										if(resolved != null && resolved.length > 0)
 											objs.addAll(Arrays.asList(resolved));
 									} catch (BuildMacroException e) {
@@ -838,34 +846,9 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 	 * @see org.eclipse.cdt.managedbuilder.core.IManagedBuildInfo#isHeaderFile(java.lang.String)
 	 */
 	public boolean isHeaderFile(String ext) {
-		IProject project = (IProject)owner;
-
 		// Check to see if there is a rule to build a file with this extension
 		IConfiguration config = getDefaultConfiguration();
-		ITool[] tools = config.getFilteredTools();
-		for (int index = 0; index < tools.length; index++) {
-			ITool tool = tools[index];
-			try {
-				// Make sure the tool is right for the project
-				switch (tool.getNatureFilter()) {
-					case ITool.FILTER_C:
-						if (project.hasNature(CProjectNature.C_NATURE_ID) && !project.hasNature(CCProjectNature.CC_NATURE_ID)) {
-							return tool.isHeaderFile(ext);
-						}
-						break;
-					case ITool.FILTER_CC:
-						if (project.hasNature(CCProjectNature.CC_NATURE_ID)) {
-							return tool.isHeaderFile(ext);
-						}
-						break;
-					case ITool.FILTER_BOTH:
-						return tool.isHeaderFile(ext);
-				}
-			} catch (CoreException e) {
-				continue;
-			}
-		}
-		return false;
+		return config.isHeaderFile(ext);
 	}
 
 	/**
@@ -951,6 +934,16 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 			// Save it
 			defaultConfig = configuration;
 			defaultConfigId = configuration.getId();
+			
+			IProject proj = getOwner().getProject();
+			IResource cdtbuildFile = proj.findMember(ManagedBuildManager.SETTINGS_FILE_NAME);
+			if(cdtbuildFile != null){
+				try {
+					cdtbuildFile.touch(new NullProgressMonitor());
+				} catch (CoreException e) {
+					//TODO: log an error
+				}
+			}
 			// TODO: is this appropriate?
 			persistDefaultConfiguration();
 		}
@@ -1145,33 +1138,66 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 	
 	/**
 	 */
-	private String processPath(String path, int context, Object obj) {
+	private List processPath(List list, String path, int context, Object obj) {
 		final String EMPTY = "";   //$NON-NLS-1$
-		final String QUOTE = "\""; //$NON-NLS-1$
-		
-		if (path == null) { return EMPTY; }
-		String s = path;
-		if (context != 0) {
-			try {
-				s = ManagedBuildManager.getBuildMacroProvider().resolveValue(s, EMPTY, " ", context, obj); //$NON-NLS-1$
-			} catch (BuildMacroException e) {
+		if (path != null) { 
+			if (context != 0) {
+				try {
+					String  paths[] = ManagedBuildManager.getBuildMacroProvider().resolveStringListValue(path, EMPTY, " ", context, obj); //$NON-NLS-1$
+					if (paths != null) {
+						for(int i = 0; i < paths.length; i++){
+							// Check for registered path converter
+							if (obj instanceof OptionContextData) {
+								OptionContextData optionContext = (OptionContextData) obj;
+								IBuildObject buildObject = optionContext.getParent() ;
+								IOptionPathConverter optionPathConverter = getPathConverter(buildObject);
+								if (null!=optionPathConverter) {
+									IPath platformPath = optionPathConverter
+											.convertToPlatformLocation(paths[i], null, null);
+									paths[i] = platformPath.toOSString();
+								}								
+							}
+							list.add(checkPath(paths[i]));
+						}
+					}
+				} catch (BuildMacroException e) {
+				}
+			} else {
+				list.add(checkPath(path));
 			}
 		}
-		if (s == null) { s = path; }
+		return list;
+	}
+	
+	private IOptionPathConverter getPathConverter(IBuildObject buildObject)  {
+		IOptionPathConverter converter = null ;
+		if (buildObject instanceof ITool) {
+			ITool tool = (ITool) buildObject;
+			converter = tool.getOptionPathConverter() ;
+		}
+		return converter ;
+	}
+	private String checkPath(String p){
+		final String QUOTE = "\""; //$NON-NLS-1$
+		final String EMPTY = "";   //$NON-NLS-1$
 
-		if (s.length()> 1 && s.startsWith(QUOTE) && s.endsWith(QUOTE)) {
-			s = s.substring(1, s.length()-1);
+		if(p == null)
+			return EMPTY;
+
+		if (p.length()> 1 && p.startsWith(QUOTE) && p.endsWith(QUOTE)) {
+			p = p.substring(1, p.length()-1);
 		}
 		
-		if ( ".".equals(s) ) { //$NON-NLS-1$
+		if ( ".".equals(p) ) { //$NON-NLS-1$
 			String cwd = getCWD();
-			if (cwd.length()>0) { s = cwd; }
+			if (cwd.length()>0) { p = cwd; }
 		}
-		if (!(new Path(s)).isAbsolute()) {
+		if (!(new Path(p)).isAbsolute()) {
 			String cwd = getCWD();
-			if (cwd.length()>0) { s = cwd + "/" + s; } //$NON-NLS-1$
+			if (cwd.length()>0) { p = cwd + "/" + p; } //$NON-NLS-1$
 		}
-		return s;
+		return p;
+		
 	}
 
 	/**
@@ -1181,7 +1207,7 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 	public IPathEntry[] getManagedBuildValues() {
 		List entries = new ArrayList();
 		int i=0;
-		IPathEntry[] a = getManagedBuildValues(IPathEntry.CDT_INCLUDE_FILE);
+		IPathEntry[] a = getManagedBuildValues(IPathEntry.CDT_INCLUDE);
 		if (a != null) { for (i=0; i<a.length; i++) entries.add(a[i]); } 
 		a = getManagedBuildValues(IPathEntry.CDT_LIBRARY);
 		if (a != null) { for (i=0; i<a.length; i++) entries.add(a[i]); } 
@@ -1197,7 +1223,7 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 	public IPathEntry[] getManagedBuildBuiltIns() {
 		List entries = new ArrayList();
 		int i=0;
-		IPathEntry[] a = getManagedBuildBuiltIns(IPathEntry.CDT_INCLUDE_FILE);
+		IPathEntry[] a = getManagedBuildBuiltIns(IPathEntry.CDT_INCLUDE);
 		if (a != null) { for (i=0; i<a.length; i++) entries.add(a[i]); } 
 		a = getManagedBuildBuiltIns(IPathEntry.CDT_LIBRARY);
 		if (a != null) { for (i=0; i<a.length; i++) entries.add(a[i]); } 
@@ -1216,9 +1242,9 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 		List entries = getOptionValues(entryType, false);
 	
 		// for includes, get env variables values; useless for other entry types  
-		if (entryType == IPathEntry.CDT_INCLUDE_FILE) {
+		if (entryType == IPathEntry.CDT_INCLUDE) {
 			IEnvironmentVariableProvider env = ManagedBuildManager.getEnvironmentVariableProvider();
-			entries = addIncludes(entries, env.getBuildPaths(getDefaultConfiguration(), IEnvVarBuildPath.BUILDPATH_INCLUDE), Path.EMPTY, null);
+			entries = addIncludes(entries, env.getBuildPaths(getDefaultConfiguration(), IEnvVarBuildPath.BUILDPATH_INCLUDE), Path.EMPTY, 0, null);
 		}	
 		return (IPathEntry[])entries.toArray(new IPathEntry[entries.size()]);
 	}
@@ -1285,7 +1311,7 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 		IPath resPath = Path.EMPTY;
 
 		// check that entryType is correct
-		if (entryType != IPathEntry.CDT_INCLUDE_FILE &&
+		if (entryType != IPathEntry.CDT_INCLUDE &&
 //TODO: we need to implement the proper CDT_LIBRARY handling
 //calculating the CDT_LIBRARY entries from the managed build 
 //options is disabled for now, we need to define a new option type
@@ -1314,20 +1340,21 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 				   !applicabilityCalculator.isOptionUsedInCommandLine(obj, t[i], op[j])) continue;
 				
 				try {
-					if (entryType == IPathEntry.CDT_INCLUDE_FILE && 
+					if (entryType == IPathEntry.CDT_INCLUDE && 
 							op[j].getValueType() == IOption.INCLUDE_PATH) 
 					{
-						OptionContextData ocd = new OptionContextData(op[j], obj);				
-						addIncludes(entries, builtIns ? op[j].getBuiltIns() : op[j].getIncludePaths(), resPath, ocd);
+						OptionContextData ocd = new OptionContextData(op[j], t[i]);				
+						addIncludes(entries, builtIns ? op[j].getBuiltIns() : op[j].getIncludePaths(), resPath, IBuildMacroProvider.CONTEXT_OPTION, ocd);
 					} else if (entryType == IPathEntry.CDT_LIBRARY && 
 							op[j].getValueType() == IOption.LIBRARIES) 
 					{
-						OptionContextData ocd = new OptionContextData(op[j], obj);				
-						addLibraries(entries, builtIns ? op[j].getBuiltIns() : op[j].getLibraries(), resPath, ocd);
+						OptionContextData ocd = new OptionContextData(op[j], t[i]);				
+						addLibraries(entries, builtIns ? op[j].getBuiltIns() : op[j].getLibraries(), resPath, IBuildMacroProvider.CONTEXT_OPTION, ocd);
 					} else if (entryType == IPathEntry.CDT_MACRO && 
 							op[j].getValueType() == IOption.PREPROCESSOR_SYMBOLS) 
 					{
-						addSymbols(entries, builtIns ? op[j].getBuiltIns() : op[j].getDefinedSymbols(), resPath);
+						OptionContextData ocd = new OptionContextData(op[j], t[i]);
+						addSymbols(entries, builtIns ? op[j].getBuiltIns() : op[j].getDefinedSymbols(), resPath, IBuildMacroProvider.CONTEXT_OPTION, ocd);
 					} else { continue; }
 				} catch (BuildException e) {}
 			}
@@ -1342,14 +1369,29 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 	 * @param resPath
 	 * @param ocd       
 	 */
-	protected List addIncludes(List entries, String[] values, IPath resPath, OptionContextData ocd) {
-		if (values != null) {
+	protected List addIncludes(List entries, String[] values, IPath resPath, int context ,Object obj) {
+		return addPaths(entries, values, resPath, context, obj, IPathEntry.CDT_INCLUDE);
+	}
+	
+	protected List addPaths(List entries, String[] values, IPath resPath, int context ,Object obj, int type){
+		if (values != null && values.length > 0) {
+			List list = new ArrayList();
 			for (int k=0; k<values.length; k++) {
-				if (ocd != null) {
-				   values[k] = processPath(values[k], IBuildMacroProvider.CONTEXT_OPTION, ocd);
+				processPath(list, values[k], context, obj);
+			}
+			
+			Iterator iter = list.iterator();
+			while(iter.hasNext()){
+				IPathEntry entry = null;
+				switch(type){
+				case IPathEntry.CDT_INCLUDE:
+					entry = CoreModel.newIncludeEntry(resPath, Path.EMPTY, new Path((String)iter.next()), true);
+					break;
+				case IPathEntry.CDT_LIBRARY:
+					entry = CoreModel.newLibraryEntry(resPath, Path.EMPTY, new Path((String)iter.next()), null, null, null, true);
+					break;
 				}
-				IPathEntry entry = CoreModel.newIncludeEntry(resPath, Path.EMPTY, new Path(processPath(values[k], 0, null)), true);
-				if (!entries.contains(entry)) {	entries.add(entry);	}
+				if (entry != null && !entries.contains(entry)) {	entries.add(entry);	}
 			}
 		}
 		return entries;
@@ -1362,17 +1404,8 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 	 * @param resPath
 	 * @param ocd
 	 */
-	protected List addLibraries(List entries, String[] values, IPath resPath, OptionContextData ocd) {
-		if (values != null) {
-			for (int k=0; k<values.length; k++) {
-				if (ocd != null) {
-					values[k] = processPath(values[k], IBuildMacroProvider.CONTEXT_OPTION, ocd);
-				}
-				IPathEntry entry = CoreModel.newLibraryEntry(resPath, Path.EMPTY, new Path(processPath(values[k], 0, null)), null, null, null, true);
-				if (!entries.contains(entry)) {	entries.add(entry); }
-			}
-		}
-		return entries;
+	protected List addLibraries(List entries, String[] values, IPath resPath, int context, Object obj) {
+		return addPaths(entries, values, resPath, context, obj, IPathEntry.CDT_LIBRARY);
 	}
 	
 	/**
@@ -1381,11 +1414,26 @@ public class ManagedBuildInfo implements IManagedBuildInfo, IScannerInfo {
 	 * @param values
 	 * @param resPath
 	 */
-	protected List addSymbols(List entries, String[] values, IPath resPath) {
+	protected List addSymbols(List entries, String[] values, IPath resPath, int context, Object obj) {
 		if (values == null) return entries;
 		for (int i=0; i<values.length; i++) {
-			if (values[i].length() == 0) continue;
-			String[] tokens = values[i].split("="); //$NON-NLS-1$
+			try {
+				String res[] = ManagedBuildManager.getBuildMacroProvider().resolveStringListValue(values[i],
+						"", " ", context, obj); //$NON-NLS-1$ //$NON-NLS-2$
+				if(res != null){
+					for(int k = 0; k < res.length; k++)
+						createMacroEntry(entries, res[k], resPath);
+				}
+			} catch (BuildMacroException e) {
+			} 
+		}
+		return entries;
+	}
+	
+	private List createMacroEntry(List entries, String val, IPath resPath){
+		if (val != null && val.length() != 0){ 
+				
+			String[] tokens = val.split("="); //$NON-NLS-1$
 			String key = tokens[0].trim();
 			String value = (tokens.length > 1) ? tokens[1].trim() : new String();
 			// Make sure the current entries do not contain a duplicate
