@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005 Intel Corporation and others.
+ * Copyright (c) 2005, 2006 Intel Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -233,7 +233,11 @@ public class NewBuildMacroDialog extends StatusDialog {
 		fListEditorContainier.setLayout(new GridLayout());
 
 		fListEditor = new FileListControl(fListEditorContainier, ManagedBuilderUIMessages.getResourceString(LIST_TITLE), IOption.BROWSE_NONE);
+		/* Enable workspace support for list editor */
+		fListEditor.setWorkspaceSupport(true);
 		
+		fListEditor.setContext(fMacrosBlock.getContextInfo());
+
 		if(fEditedMacro != null){
 			loadMacroSettings(fEditedMacro,true);
 			fMacroNameEdit.setEnabled(false);
